<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version
 * @author Federico Tarantini federicot@gmail.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * Purge old db entries cronjob
 */

require_once("system/Cronjob.php");
require_once('core/ActivityLog.php');
require_once('core/FilesystemPeer.php');
require_once('core/UserSessionPeer.php');
require_once('core/UserPreferencePeer.php');

class PurgeOldEntries extends Cronjob {
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}

	public function run() {
		$this->purge_activity_log();
		$this->purge_user_accounts();
		$this->purge_deleted_files();
		$this->purge_user_sessions();
		$this->purge_cron_history();
		$this->purge_user_preferences();
	}

	//filesystem entries that are marked deleted by fules
	public function purge_deleted_files() {
		$crit=new Criteria();
		$crit->add(FilesystemPeer::DELETED,true);
		$files=FilesystemPeer::doSelect($crit);
		foreach ($files as $file) {
			$file->delete();
		}
	}

	//Purges older than 30 days activity_log table entries 
	public function purge_activity_log() {
		$now = time();
		$m30days = $now-2592000; //today minus 30 days (2592000 seconds)
		$date = date('Y-m-d h:i:s', $m30days);
		
		$crit = new Criteria();
		$crit->add(ActivityLogPeer::ACTIVITY_DATE, $date, Criteria::LESS_EQUAL);
		ActivityLogPeer::doDelete($crit);
	}

	//Purges useraccounts that have STAUS_PENDING and that are 30 days old
	public function purge_user_accounts() {
		$now = time();
		$m30days = $now-2592000; //today minus 30 days (2592000 seconds)

		$crit = new Criteria();
		$crit->add(UserPeer::STATUS,User::STATUS_PENDING);
		$crit->add(UserPeer::CREATE_DATE, date('Y-m-d h:i:s', $m30days), Criteria::LESS_EQUAL);
		$sel_user = UserPeer::doDelete($crit);
		
		UserPeer::doDelete($crit);
	}

	//purge login records after 30 days
	public function purge_user_sessions() {
		$now = time();
		$m30days = $now-2592000; //today minus 30 days (2592000 seconds)

		$crit = new Criteria();
		$crit->add(UserSessionPeer::LOGIN_TIME, date('Y-m-d h:i:s', $m30days), Criteria::LESS_EQUAL);
		$sel_user = UserSessionPeer::doDelete($crit);
		
		UserPeer::doDelete($crit);
	}

	//purge all cron activity after 30 days
	public function purge_cron_history() {
		$now = time();
		$m30days = $now-2592000; //today minus 30 days (2592000 seconds)

		$crit = new Criteria();
		$crit->add(CronHistoryPeer::RUNDATE, date('Y-m-d h:i:s', $m30days), Criteria::LESS_EQUAL);
		$sel_user = CronHistoryPeer::doDelete($crit);
		
		UserPeer::doDelete($crit);
	}

	//purge all user preferences older than 90 days
	public function purge_user_preferences() {
		$now = time();
		$m90days = $now-7776000; //today minus 90 days (7776000 seconds)
		$date = date('Y-m-d h:i:s', $m90days);
		$crit = new Criteria();
		$crit->add(UserPreferencePeer::SET_DATE, date('Y-m-d h:i:s', $m90days), Criteria::LESS_EQUAL);

		UserPreferencePeer::doDelete($crit);
	}
}

$cronjob = new PurgeOldEntries();
$cronjob->run();
?>
