<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-03-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/docman/lib/om/BaseDocmanFolder.php';
require_once("plugin/ActionManager.php");
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");

/** 
 * The skeleton for this class was autogenerated by Propel  
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.docman.lib 
 */
class DocmanFolder extends BaseDocmanFolder implements GFPermission {
	const SECTION='docmanfolder';
	const PERMLEVEL_NOACCESS=0;
	const PERMLEVEL_READ=1;
	const PERMLEVEL_WRITE=2;
	const PERMLEVEL_ADMIN=3;
	const IS_PUBLIC=1;
	const IS_PRIVATE=0;

	function getId() {
		return $this->getDocmanFolderId();
	}

	function getName() {
		return $this->getFolderName();
	}
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see DocmanFolder::setDeleted()
	 * @see DocmanFolder::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException();
		}
		
		$crit = new Criteria();
		$crit->add(DocmanFolderPeer::PARENT_FOLDER_ID,$this->getDocmanFolderId());
		$folders = DocmanFolderPeer::doSelect($crit);
		foreach ($folders as $folder) {
			$folder->delete();
		}
		
		// Audit trail
		$audit = new AuditTrail();
		$audit->setUser(Session::getSession()->getUser());
		$audit->setSection(Project::SECTION);
		$audit->setRefId($this->getProjectId());
		$audit->setFieldName(self::SECTION);
		$audit->setNewValue('_deleted_');
		$audit->setOldValue($this->getFolderName().' '.Language::getSessText('Docman.FolderDeleteAllContents'));
		$audit->save();		
		
		ActionManager::notify("DOCMAN_FOLDER_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null, $check_duplicity = true) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException();
		} else if (!$this->isNew() && !$this->userCanUpdate()) {
			throw new PermissionDeniedException();
		}

		$errors=array();
		if (strlen($this->folder_name) < 1) {
			$errors['folder_name'] = Language::getSessText('Docman.EmptyFoldername');
		}
		// first check. if they want to make the same folder name under same parent, don't allow it
		if ($this->isNew() && (count($errors) == 0) && $check_duplicity) {
			$c = new Criteria();
			$c->add(DocmanFolderPeer::FOLDER_NAME,$this->folder_name);
			$c->add(DocmanFolderPeer::PARENT_FOLDER_ID,$this->parent_folder_id);
			$c->add(DocmanFolderPeer::PROJECT_ID,$this->getProjectId());
			$already_here = DocmanFolderPeer::doSelectOne($c);
			if (is_object($already_here)) {
				$errors['folder_name_exists'] = Language::getSessText('Docman.FolderExists');
			}
		}
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}

		if (($this->isNew()) && ($this->parent_folder_id == 0)) {
			$createrolesettings = true;
		} else {
			$createrolesettings = false;
		}

		parent::save($con);
		ActionManager::notify("DOCMAN_FOLDER_SAVE", $this);

		if ($createrolesettings) {
			$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
			$sql="INSERT INTO role_setting (role_id,section,ref_id,value) 
				SELECT role_id,'".self::SECTION."','".$this->getPrimaryKey()."',
				'".self::PERMLEVEL_READ."' 
				FROM role WHERE project_id='".$this->getProject()->getProjectId()."'";
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql);
		}


	}


	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		//we are now only setting this on the root folder
		return $this->getRootFolder()->getIsPublic();
		/*
		//start with checking if the project is public or not
		$parent_public=$this->getProject()->isPublic();

		//then check if this is a subfolder - if it is the parent may be private, so this is private
		if ($this->getParentFolderId() > 0) {
			$parent=DocmanFolderPeer::retrieveByPk($this->getParentFolderId());
			if (is_object($parent)) {
				$parent_public=$parent_public && $parent->isPublic();
			} else {
				echo "ERROR - INVALID PARENT ID SET ON FOLDER: ".$this->getDocmanFolderId();
			}
		}
		return $this->getIsPublic() && $parent_public;
		*/
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}
		if ( ($this->getParentFolderId()==0) && ($this->getFolderName()=='incoming') ) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		$perm = $this->getProject()->getPermission();
		if ($perm->check(DocmanPlugin::SECTION, 0, DocmanPlugin::ADMIN) || $perm->check(DocmanFolder::SECTION, $this->getRootFolder()->getDocmanFolderId(), DocmanFolder::PERMLEVEL_ADMIN)) {
			return true;
		}
		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 */
	public function userCanRead() {
		if ($this->getParentFolderId() > 0) {
			//if this is a subfolder, user the parent folder perms
			$parent=DocmanFolderPeer::retrieveByPk($this->getParentFolderId());
			if (is_object($parent)) {
				return $parent->userCanRead();
			} else {
				echo "ERROR - INVALID PARENT ID SET ON FOLDER: ".$this->getDocmanFolderId();
			}
		} else {
			if ($this->isPublic()) {
				return true;
			}
			if ($this->userIsAdmin()) {
				return true;
			}
	//COMMUNITY EXPRESS
			if (Session::getSession()->isLoggedIn() && Session::getSession()->getUser()->isProjectMember($this->getProject())) {
				return true;
			}
			/*
			$perm = $this->getProject()->getPermission();
			// check if the user has permissions for reading at least
			if ($perm->check(DocmanFolder::SECTION, $this->getDocmanFolderId(), DocmanFolder::PERMLEVEL_READ)) {
				return true;
			}*/
			return false;
		}
	}

	/**
	 * Returns the root folder if this is not the root, otherwise returns itself
	 *
	 * @return bool
	 */
	public function getRootFolder() {
		if ($this->getParentFolderId() > 0) {
			//if this is a subfolder, user the parent folder perms
			$parent=DocmanFolderPeer::retrieveByPk($this->getParentFolderId());
			if (is_object($parent)) {
				return $parent->getRootFolder();
			} else {
				echo "ERROR - INVALID PARENT ID SET ON FOLDER: ".$this->getDocmanFolderId();
			}
		} else {
			return $this;
		}
	}
}
?>
