<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli	<marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('plugin/PluginResult.php');
require_once('plugins/scm/lib/ScmPlugin.php');
require_once('core/Language.php');


class ScmperforcePlugin extends ScmPlugin {
	function __construct() {
	}	
	
	public function getType() {
		return GFPlugin::SCM_PLUGIN;
	}
	
	public function getListeners() {
		$listeners = parent::getListeners();
		$listeners[] = 'GET_SCM_SERVER_TYPES';
		$listeners[] = 'GET_PERFORCEADMIN_NAV_ELEMENTS';
		$listeners[] = 'USER_LOGIN';
		return $listeners;
	}	
	
	function handleAction($listenername,$params) {
		switch ($listenername) {
			case 'GET_SCM_SERVER_TYPES':
				return $this->getScmServerType($params); 
				break;
			case 'USER_LOGIN':
				$this->updateLoginData($params);
				break;
			case 'GET_PERFORCEADMIN_NAV_ELEMENTS':
				return $this->getScmAdminNavElements($params); 
				break;
			default:
				return parent::handleAction($listenername, $params);
				break;
		}
	}
	
	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	public function getName() {
		return 'scmperforce';
	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return PluginResult
	 */
	public function search(GFSearch $search) {
	}
	
	
	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->projectURL($this->project,'','scmperforce');
	}

	protected function buildMainTabsListener($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('Perforce.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','scmperforce'),
				'id'	=> 'scmperforce'
			);
		$result->setResult($value);
		return $result;
	}

	/**
	 *      getScmNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	protected function getScmNavElements($params) {
		$lang = Language::getSessLang();
		
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];		
		
		$elements = array(
						array(
							'link'=> GFUrl::getUrl()->projectUrl($project,'AccessInfo','scmperforce'),
							'description' => $lang->getText('Perforce.AccessInfo')
						),
						array(
							'link'=> GFUrl::getUrl()->projectUrl($project,'browse','scmperforce'),
							'description' => $lang->getText('Perforce.Browse')
						),
						array(
							'link'=> GFUrl::getUrl()->projectUrl($project,'','scmsvn'),
							'description' => $lang->getText('Scmsvn.Browse')
						),						
						array(
							'link'=> 'http://www.perforce.com/perforce/technical.html#usermanuals',
							'description' => $lang->getText('Perforce.ReferenceTitle'),
							'target' => '_blank'
						),
					);
				
		if ($loggedin) {
			if ($project->getPermission()->isAdmin()) {
				$elements[] = array('link' => $url->projectURL($project,'','scmperforce','admin', ''),
					'description' => $lang->getText('Tabs.Admin'));
			}
		}				
		
		$result = new PluginResult($this);
		$arr = array(
			'title'=> 'Perforce',
			'elements'=> $elements
			);
		$result->setResult($arr);
		return $result;
	}
	
	/**
	 * 	The getObserverRoleChoices listener - this hook will fire
	 *	when the observer role is modified.
	 * 	It returns an array of role settings or null
	 *	@return	array or null
	 */
	function getObserverRoleChoices($params) {	
		return parent::getObserverRoleChoices($params);
	}
	
	/**
	 * Retrieves the concerning data and updates its observer role settings
	 *
	 * @param array	$data
	 */	
	function updateObserverSettings($data) {	
		return parent::updateObserverSettings($data);
	}		

	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	protected function getProjectAdminNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$lang = Session::getSession()->getLanguage();
		$url = GFUrl::getUrl();

		$result = new PluginResult($this);
		$arr = array(
			'title'=> $lang->getText('Perforce.TabString'),
			'elements'=> array(
			array('link'=> $url->projectUrl($params['project'], '', $this->getName(), 'admin'),
				'description'=> $lang->getText('Perforce.Admin'))
			)
		);
		$result->setResult($arr);
		return $result;
	}
	

	protected function getScmAdminNavElements($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];		
		
		$elements = array(
						array(
							'link'=> $url->projectUrl($project,'ScmModuleBrowse','scmperforce', 'admin'),
							'description' => $lang->getText('Perforce.BrowseModules')
						),
					);
				
		$result = new PluginResult($this);
		$arr = array(
			'title'=> $lang->getText('Perforce.AdminNavTitle'),
			'elements'=> $elements
			);
		$result->setResult($arr);
		return $result;
	}

	protected function getScmServerType($params) {
		$pr = new PluginResult($this);
		$pr->setResult('perforce');
		return $pr;
	}
	
	protected function updateLoginData($params) {
		$username = $params['username'];
		$password = $params['password'];
		
		// We must add the user to the Perforce server. This is not done in a cronjob
		// because we need the plain password, so we do this here.
		require_once('plugins/scmperforce/lib/PerforceManager.php');
		require_once('plugins/scm/lib/ProjectScmPeer.php');
		require_once('core/UserProjectPeer.php');
		require_once('core/UserPeer.php');
		require_once('core/ProjectPluginPeer.php');
		require_once('core/PluginPeer.php');
		require_once('plugins/scmperforce/lib/PerforceConfigFile.php');
		require_once('plugins/scmperforce/lib/PerforceManager.php');
		
		// get the user object
		$user = UserPeer::retrieveByUnixName($username);
		if (!is_object($user)) {
			return;
		}
		
		// Only add the user if it belongs to a managed project
		$c = new Criteria();
		$c->add(UserProjectPeer::USER_ID, $user->getUserId());
		$c->addJoin(ProjectScmPeer::PROJECT_ID, UserProjectPeer::PROJECT_ID);
		$c->addJoin(ProjectPluginPeer::PROJECT_ID, ProjectScmPeer::PROJECT_ID);
		$c->addJoin(PluginPeer::PLUGIN_ID, ProjectPluginPeer::PLUGIN_ID);
		$c->add(ProjectScmPeer::CONTROLLED, true);
		$c->add(PluginPeer::PLUGIN_NAME, $this->getName());
		
		$n = ProjectScmPeer::doCount($c);
		if ($n == 0) {
			return;
		}
		
		// In this point, we know the user belongs to at least one managed
		// project which uses Perforce. We must add it to the Perforce server
		// or update his information.
		// Create the update file
		$file = tempnam('/tmp', 'gf_perforce');
		try {
			$config = new PerforceConfigFile($file);
			$config->setVar('user', $username);
			$config->setVar('password', $password);
			$config->setVar('fullname', $user->getFullName());
			$config->setVar('email', $user->getEmail());
			$config->save();
			
			$manager = new PerforceManager();
			$manager->setUser($file);
		} catch (Exception $e) {
			// user could not be updated -- not a fatal error, may be fixed later
		}
		unlink($file);
	}
}

?>
