<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling CruisecontrolDefect

require_once('plugins/build/lib/CruisecontrolDefect.php');
require_once('plugins/build/lib/CruisecontrolDefectPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'CruisecontrolDefect',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'cruisecontrol_defect_id' => array('name'=>'cruisecontrol_defect_id', 'type' => 'xsd:int'),
		'cruisecontrol_id' => array('name'=>'cruisecontrol_id', 'type' => 'xsd:int'),
		'tracker_item_id' => array('name'=>'tracker_item_id', 'type' => 'xsd:int'),
		'open' => array('name'=>'open', 'type' => 'xsd:boolean'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfCruisecontrolDefect',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:CruisecontrolDefect[]')),
	'tns:CruisecontrolDefect'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getCruisecontrolDefects',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
		'tracker_item_id' => 'xsd:int',
	), 
	array('getCruisecontrolDefects' => 'tns:ArrayOfCruisecontrolDefect'),
	$uri, 
	$uri.'#getCruisecontrolDefects',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of CruisecontrolDefect
 * @param	$cruisecontrol_id	Filter by cruisecontrol_id
 * @param	$tracker_item_id	Filter by tracker_item_id
 */
function getCruisecontrolDefects($sessionHash, $cruisecontrol_id, $tracker_item_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$crit = new Criteria();

	if (!is_null($cruisecontrol_id) && $cruisecontrol_id > 0) {
		$crit->add(CruisecontrolDefectPeer::CRUISECONTROL_ID, $cruisecontrol_id);
	}
	if (!is_null($tracker_item_id) && $tracker_item_id > 0) {
		$crit->add(CruisecontrolDefectPeer::TRACKER_ITEM_ID, $tracker_item_id);
	}
	
	$objs = CruisecontrolDefectPeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = CruisecontrolDefectToSOAP($obj);
	  }
	}
	return $res;
}

$server->register(
	'getCruisecontrolDefectArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_defect_id' => 'ArrayOfxsd:int',
	), 
	array('getCruisecontrolDefect' => 'tns:ArrayOfCruisecontrolDefect'),
	$uri, 
	$uri.'#getCruisecontrolDefect',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of CruisecontrolDefect
 * @param	$cruisecontrol_defect_id	Retrieve the CruisecontrolDefect array identified by the array of cruisecontrol_defect_id
 */
function getCruisecontrolDefectArray($sessionHash, $cruisecontrol_defect_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$ret = array();

	$crit=new Criteria();
        $crit->add(CruisecontrolDefectPeer::CRUISECONTROL_DEFECT_ID,$cruisecontrol_defect_idArr,Criteria::IN);
	$objs = CruisecontrolDefectPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = CruisecontrolDefectToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getCruisecontrolDefect',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_defect_id' => 'xsd:int',
	), 
	array('getCruisecontrolDefect' => 'tns:CruisecontrolDefect'),
	$uri, 
	$uri.'#getCruisecontrolDefect',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single CruisecontrolDefect
 * @param	$cruisecontrol_defect_id	Retrieve the CruisecontrolDefect identified by this cruisecontrol_defect_id
 */
function getCruisecontrolDefect($sessionHash, $cruisecontrol_defect_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolDefectPeer::retrieveByPK($cruisecontrol_defect_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getCruisecontrolDefect','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return CruisecontrolDefectToSOAP($obj);
}


$server->register(
	'deleteCruisecontrolDefect',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_defect_id' => 'xsd:int',
	), 
	array('deleteCruisecontrolDefect' => 'xsd:int'),
	$uri, 
	$uri.'#deleteCruisecontrolDefect',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single CruisecontrolDefect
 * @param	$cruisecontrol_defect_id	Delete the CruisecontrolDefect identified by this cruisecontrol_defect_id
 */
function deleteCruisecontrolDefect($sessionHash, $cruisecontrol_defect_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolDefectPeer::retrieveByPK($cruisecontrol_defect_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteCruisecontrolDefect','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteCruisecontrolDefect','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateCruisecontrolDefect',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_defect_id' => 'xsd:int',
		'cruisecontrol_id' => 'xsd:int',
		'tracker_item_id' => 'xsd:int',
		'open' => 'xsd:boolean',
	), 
	array('updateCruisecontrolDefect' => 'xsd:int'),
	$uri, 
	$uri.'#updateCruisecontrolDefect',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update CruisecontrolDefect
 * @param	$cruisecontrol_defect_id	 CruisecontrolDefect to update
 * @param	$cruisecontrol_id
 * @param	$tracker_item_id
 * @param	$open
 */
function updateCruisecontrolDefect($sessionHash, $cruisecontrol_defect_id, $cruisecontrol_id, $tracker_item_id, $open) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolDefectPeer::retrieveByPK($cruisecontrol_defect_id);
	if (is_null($obj)) {
		throw new soap_fault('', 'updateCruisecontrolDefect', 'Couldn\'t update CruisecontrolDefect: invalid cruisecontrol_defect_id '.$cruisecontrol_defect_id);
	}
	
	if (!is_null($cruisecontrol_id)) {
		$obj->setCruisecontrolId( Validator::sanitizeInt($cruisecontrol_id) );
	}
	if (!is_null($tracker_item_id)) {
		$obj->setTrackerItemId( Validator::sanitizeInt($tracker_item_id) );
	}
	if (!is_null($open)) {
		$obj->setOpen( Validator::sanitizeBool($open) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateCruisecontrolDefect','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addCruisecontrolDefect',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
		'tracker_item_id' => 'xsd:int',
		'open' => 'xsd:boolean',
	), 
	array('addCruisecontrolDefect' => 'xsd:int'),
	$uri, 
	$uri.'#addCruisecontrolDefect',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a CruisecontrolDefect
 * @param	$cruisecontrol_id
 * @param	$tracker_item_id
 * @param	$open
 */
function addCruisecontrolDefect($sessionHash, $cruisecontrol_id, $tracker_item_id, $open) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = new CruisecontrolDefect();
	
	$obj->setCruisecontrolId( Validator::sanitizeInt($cruisecontrol_id) );
	$obj->setTrackerItemId( Validator::sanitizeInt($tracker_item_id) );
	$obj->setOpen( Validator::sanitizeBool($open) );
	try {
		$obj->save();
		return $obj->getPrimaryKey();
	} catch (Exception $e) {
		return new soap_fault('', 'addCruisecontrolDefect','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function CruisecontrolDefectToSOAP(&$obj) {
	$ret = array();
	$ret['cruisecontrol_defect_id'] = $obj->getCruisecontrolDefectId();
	$ret['cruisecontrol_id'] = $obj->getCruisecontrolId();
	$ret['tracker_item_id'] = $obj->getTrackerItemId();
	$ret['open'] = $obj->getOpen();
	return $ret;
}
?>
