<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.com
 * @date 2006-10-10
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/ProjectPeer.php');
require_once('session/Session.php');
require_once('plugins/tracker/lib/TrackerExtraField.php');
require_once('html/HTMLWidget.php');

$session = Session::getSession();
$project = GFUrl::getUrl()->getProject();
$project_id = $project->getProjectId();
$lang = $session->getLanguage();

if (!isset($feedback)) {
	$feedback = '';
}
require_once('html/templates/ProjectReportingTemplate.php');

$template = new ProjectReportingTemplate();
$params = array(
		'project' => $project,
		'feedback' => $feedback,
		'title'=>$lang->getText('FrsRelease.DevelopmentRoadmap')
	);
$template->header($params);

// Get connection
$con = Propel::getConnection(UserPeer::DATABASE_NAME);
$stmt = $con->createStatement();

// Get release_id
$pkg_rls_id = ((isset($_REQUEST['pkg_rls_id'])) ? $_REQUEST['pkg_rls_id'] : '0,0' );
list ($package_id, $release_id) = explode(',', $pkg_rls_id);

$package_id = Validator::sanitizeInt($package_id);
$release_id = Validator::sanitizeInt($release_id);

$subprojects=((isset($_REQUEST['subprojects'])) ? Validator::sanitizeInt($_REQUEST['subprojects']) : 0 );

$releases = array();
$releases['0,0'] = $lang->getText('Reporting.FutureReleases');

// Get all Project Ids and Names
$all_ids = array();
$all_names = array();
if ($subprojects == 1) {
	$all_ids = $project->getAllSubprojectIds();
	foreach ($project->getAllSubprojects() as $subPrj) {
		$all_names[$subPrj->getProjectId()] = $subPrj->getProjectName();
		$subPrjs[$subPrj->getProjectId()] = $subPrj;
	}
	$all_ids[] = $project_id;
	$all_names[$project->getProjectId()] = $project->getProjectName();
} else {
	$all_ids[] = $project_id;
}

$sql = "SELECT frsp.frs_package_id, frsr.frs_release_id,
frsp.package_name || '::' || frsr.release_name as release, frsp.project_id
FROM frs_package frsp, frs_release frsr
WHERE frsr.frs_package_id=frsp.frs_package_id 
AND frsp.project_id IN (".implode(',',$all_ids).") 
ORDER BY frsp.package_name,frsr.release_name;";

$rs = $stmt->executeQuery($sql);

if ($rs != null && $rs->getRecordCount() > 0) {
	while($rs->next()) {
		$pkgid = $rs->getInt('frs_package_id');
		$rlsid = $rs->getInt('frs_release_id');
		$prjid = $rs->getInt('project_id');
		if ($prjid != $project_id) {
			$name = $all_names[$prjid] . ' > ' . $rs->getString('release');
		} else {
			$name = $rs->getString('release');
		}
		$releases["$pkgid,$rlsid"] = $name;
	}
}
?>
<form action="<?php echo GFUrl::getUrl()->currentURL() ?>" method="post">
<table border="0" cellspacing="0" cellpadding="3" bgcolor="#E0E0E0">
<tr>
	<td><strong><?php echo $lang->getText('Reporting.SelectRelease') ?></strong></td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td><?php echo HTMLWidget::selectBoxFromAssoc($releases, 'pkg_rls_id', $pkg_rls_id, false, false) ?></td>
	<td valign="top"><?php echo HTMLWidget::button("submit", $lang->getText('Common.Submit'), "submit"); ?></td>
</tr>
<tr>
	<td>
		<strong><?php echo $lang->getText('ProjectReporting.IncludeSubprojects') ?></strong> 
		<input type="checkbox" name="subprojects" value="1"<?php echo ($subprojects == 1) ? ' checked' : '';?>/>
	</td>
	<td>&nbsp;</td>
</tr>
</table>
</form>
<br />
<table>
	<tr>
		<td>
			<?php echo "<img src='/gf/project/".$project->getUnixName()."/graphs/?action=ItemsFoundInRelease&pkg_rls_id=$pkg_rls_id&subprojects=$subprojects' alt='' />"; ?>
		</td>
		<td>
			<?php echo "<img src='/gf/project/".$project->getUnixName()."/graphs/?action=ItemsFixedInRelease&pkg_rls_id=$pkg_rls_id&subprojects=$subprojects' alt='' />"; ?>
		</td>
	</tr>
</table>
<br />
<?php

$sql = "SELECT frsp.package_name, frsp.frs_package_id,
frsr.release_name,frsr.frs_release_id, t.tracker_name,t.tracker_id,
tef.tracker_extra_field_id,tef.field_name,ti.tracker_item_id,
ti.priority,ti.summary,ti.status_id,frsp.project_id
FROM frs_package frsp, 
frs_release frsr,
tracker_extra_field tef,
tracker_extra_field_data tefd,
tracker t,
tracker_item ti
WHERE frsr.frs_package_id=frsp.frs_package_id
AND frsp.project_id IN (".implode(',',$all_ids).") 
AND frsr.frs_release_id=tefd.field_data
AND tefd.tracker_extra_field_id=tef.tracker_extra_field_id
AND tefd.tracker_item_id=ti.tracker_item_id
AND ti.tracker_id=t.tracker_id
AND tef.field_type='".TrackerExtraField::TYPE_RELEASE."'";

if ($pkg_rls_id == '0,0') {
	$sql .= " AND frsr.is_released=false ";
} else {
	$sql .= " AND frsr.frs_package_id='$package_id' AND frsr.frs_release_id = '$release_id' ";
}
$sql .= "ORDER BY frsp.package_name,frsr.release_name,t.tracker_id,tef.field_name,ti.tracker_item_id";

$rs = $stmt->executeQuery($sql);

$opencount = array();
$closedcount = array();
$trackercount = array();
$fieldcount = array();
$packages = array();

if ($rs != null && $rs->getRecordCount() > 0) {
	while($rs->next()) {
		$pkgid = $rs->getInt('frs_package_id');
		$rlsid = $rs->getInt('frs_release_id');
		$trkid = $rs->getInt('tracker_id');
		$fieldid = $rs->getInt('tracker_extra_field_id');
		$statusid = $rs->getInt('status_id');

		// Initialize arrays
		if (!isset($opencount[$pkgid])) {
			$opencount[$pkgid] = array();
			$closedcount[$pkgid] = array();
			$trackercount[$pkgid] = array();
			$fieldcount[$pkgid] = array();
		}
		// Initialize more arrays
		if (!isset($opencount[$pkgid][$rlsid])) {
			$opencount[$pkgid][$rlsid] = 0;
			$closedcount[$pkgid][$rlsid] = 0;
			$trackercount[$pkgid][$rlsid] = array();
			$fieldcount[$pkgid][$rlsid] = array();
		}
		
		// Initialize and/or set tracker count
		if (!isset($trackercount[$pkgid][$rlsid][$trkid])) {
			$trackercount[$pkgid][$rlsid][$trkid] = 0;
		}
		$trackercount[$pkgid][$rlsid][$trkid]++;
		
		// Initialize and/or set field count
		if (!isset($fieldcount[$pkgid][$rlsid][$fieldid])) {
			$fieldcount[$pkgid][$rlsid][$fieldid] = 0;
		}
		$fieldcount[$pkgid][$rlsid][$fieldid]++;

		// set status id count
		if ($statusid == 1) {
			$opencount[$pkgid][$rlsid]++;
		} else {
			$closedcount[$pkgid][$rlsid]++;
		}
		// add to packages array
		$packages = addToArray($packages, $rs);
	}
} else {
	$theader=array();
	$theader[] = $lang->getText('Tracker.TrackerNameName');
	$theader[] = $lang->getText('TrackerExtraField.FieldNameName');
	$theader[] = $lang->getText('TrackerItem.SummaryName');
	$theader[] = $lang->getText('TrackerItem.PriorityName');
	$theader[] = $lang->getText('TrackerItem.StatusIdName');
	echo $template->tableTop($theader);
	echo $template->tableFullRow(Language::getSessText('FrsRelease.NoAssociatedItems'));
	echo $template->tableBottom();
}

$lastpackage = 0;
$lastrelease = 0;
$lasttracker = 0;
$lastfield = 0;
$tableopen = false;

foreach ($packages as $package) {
	$prjid = $package['prjid'];
	$pkgid = $package['pkgid'];
	$rlsid = $package['rlsid'];
	$trkid = $package['trkid'];
	$itemid = $package['itemid'];
	$fieldid = $package['fieldid'];

	if ($lastpackage != $pkgid) {
		$lastpackage = $pkgid;
		if ($tableopen) {
			echo $template->tableBottom();
			echo '<p>&nbsp;</p>';
			$tableopen = false;
		}
		if ($subprojects == 1) {
			if(isset($subPrjs[$prjid]))
				echo '<a href="'. GFUrl::getUrl()->projectUrl($subPrjs[$prjid],'FrsReleaseBrowse','frs').'&frs_package_id='.$pkgid.'"><h3><img src="/images/package_icon.gif" height="16" width="16" border="0" /> '.$all_names[$prjid].' &gt; '.$package['pkgname'].'</h3></a>';
			else
				echo '<a href="'. GFUrl::getUrl()->projectUrl($project,'FrsReleaseBrowse','frs').'&frs_package_id='.$pkgid.'"><h3><img src="/images/package_icon.gif" height="16" width="16" border="0" /> '.$all_names[$prjid].' &gt; '.$package['pkgname'].'</h3></a>';
		} else {
			echo '<a href="'. GFUrl::getUrl()->projectUrl($project,'FrsReleaseBrowse','frs').'&frs_package_id='.$pkgid.'"><h3><img src="/images/package_icon.gif" height="16" width="16" border="0" /> '.$package['pkgname'].'</h3></a>';
		}
	}
	if ($lastrelease != $rlsid) {
		$lastrelease = $rlsid;
		$lasttracker = 0;
		$lastfield = 0;
		if ($tableopen) {
			echo $template->tableBottom();
			echo '<p>&nbsp;</p>';
			$tableopen = false;
		}
		echo '<p><a href="'. GFUrl::getUrl()->projectUrl($project,'FrsReleaseView','frs').'&release_id='.$rlsid.'"><b>'.$package['rlsname'].'</b></a></p>';

		$total = $opencount[$pkgid][$rlsid] + $closedcount[$pkgid][$rlsid];
		$closed = $closedcount[$pkgid][$rlsid];
		drawProgressBar($total, $closed);

		$theader=array();
		$theader[] = $lang->getText('Tracker.TrackerNameName');
		$theader[] = $lang->getText('TrackerExtraField.FieldNameName');
		$theader[] = $lang->getText('TrackerItem.SummaryName');
		$theader[] = $lang->getText('TrackerItem.PriorityName');
		$theader[] = $lang->getText('TrackerItem.StatusIdName');
		echo $template->tableTop($theader);
		$tableopen = true;
	}
	$trow=array();
	$tmod = array();
	if ($lasttracker != $trkid) {
		$tmod[]='rowspan="'.$trackercount[$pkgid][$rlsid][$trkid].'" bgcolor="#FFFFFF"';		
		if(isset($subPrjs[$prjid]))			
			$trow[]='<a href="'. GFUrl::getUrl()->projectUrl($subPrjs[$prjid],'TrackerItemBrowse','tracker').'&tracker_id='.$trkid.'">'.$package['trkname'].'</a>';
		else		
			$trow[]='<a href="'. GFUrl::getUrl()->projectUrl($project,'TrackerItemBrowse','tracker').'&tracker_id='.$trkid.'">'.$package['trkname'].'</a>';
		$lasttracker = $trkid;
	}
	if ($lastfield != $fieldid) {
		$tmod[]='rowspan="'.$fieldcount[$pkgid][$rlsid][$fieldid].'" bgcolor="#FFFFFF"';
		$trow[]=$package['fieldname'];
		$lastfield = $fieldid;
	}

	$tmod[]='';
	if(isset($subPrjs[$prjid]))			
		$trow[]='<a href="'. GFUrl::getUrl()->projectUrl($subPrjs[$prjid],'TrackerItemEdit','tracker').'&tracker_id='.$trkid.'&amp;tracker_item_id='.$itemid .'">[#'.$itemid.'] '.$package['summary'].'</a>';
	else
		$trow[]='<a href="'. GFUrl::getUrl()->projectUrl($project,'TrackerItemEdit','tracker').'&tracker_id='.$trkid.'&amp;tracker_item_id='.$itemid .'">[#'.$itemid.'] '.$package['summary'].'</a>';
	$tmod[]='bgcolor="'.$theme->getPriorityColor($package['priority']).'"';
	$trow[]=$package['priority'];
	$tmod[]='';
	$trow[]=($package['status'] == 1) ? $lang->getText('tracker.status.open') : $lang->getText('tracker.status.closed');

	echo $template->tableRow($trow, $tmod);
	unset($trow);
	unset($tmod);
}
echo $template->tableBottom();

function addToArray($packages, $rs) {
	$prjid = $rs->getInt('project_id');
	$pkgid = $rs->getInt('frs_package_id');
	$pkgname = $rs->getString('package_name');
	$rlsid = $rs->getInt('frs_release_id');
	$rlsname = $rs->getString('release_name');
	$trkid = $rs->getInt('tracker_id');
	$trkname = $rs->getString('tracker_name');
	$fieldid = $rs->getInt('tracker_extra_field_id');
	$fieldname = $rs->getString('field_name');
	$itemid = $rs->getInt('tracker_item_id');
	$priority = $rs->getString('priority');
	$summary = $rs->getString('summary');
	$statusid = $rs->getInt('status_id');

	$packages[] = array('prjid'=>$prjid,'pkgid'=>$pkgid,'pkgname'=>$pkgname,'rlsid'=>$rlsid,
					'rlsname'=>$rlsname,'trkid'=>$trkid,'trkname'=>$trkname,
					'itemid'=>$itemid,'fieldid'=>$fieldid, 'fieldname'=>$fieldname,
					'summary'=>$summary,'priority'=>$priority,'status'=>$statusid);
	return $packages;
}

function drawProgressBar($total, $closed) {
	$val01 = round(($closed * 100) / $total);
	$val02 = 100 - $val01;
	echo '<table width="300" border="0"><tr>';
	echo '<td>'. Language::getSessText('Reporting.Progress'). ':</td><td>';
	echo '<table width="200" border="0" cellspacing="1" style="border: 1px solid #000"><tr>';
	if ($val01 > 99) {
		echo '<td width="100%" bgcolor="green">&nbsp;</td>';
	} else if ($val01 < 1) {
		echo '<td width="100%" bgcolor="red">&nbsp;</td>';
	} else {
		echo '<td width="'.$val01.'%" bgcolor="green">&nbsp;</td>';
		echo '<td width="'.$val02.'%" bgcolor="red">&nbsp;</td>';
	}
	echo '</tr></table>';
	echo '</td><tr><td colspan="2">';
	echo Language::getSessText('Reporting.ProgressText', array($closed, $total));
	echo '</td></tr></table>';
}

$template->footer();
?>
